﻿using System;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Imaging;
using System.Windows.Threading;
using ThinkGeo.MapSuite.SilverlightEdition;

namespace Post5716
{
    public partial class Page : UserControl
    {
        Collection<Feature> features;

        public Page()
        {
            InitializeComponent();

            Loaded += new RoutedEventHandler(Page_Loaded);
        }

        void Page_Loaded(object sender, RoutedEventArgs e)
        {
            DispatcherTimer timerOpen = new DispatcherTimer();
            timerOpen.Interval = TimeSpan.FromSeconds(4);
            timerOpen.Tick += new EventHandler(timerOpen_Tick);

            DispatcherTimer timerJammed = new DispatcherTimer();
            timerJammed.Interval = TimeSpan.FromSeconds(2);
            timerJammed.Tick += new EventHandler(timerJammed_Tick);

            features = GetFeatures();

            SimpleMarkerOverlay markers = new SimpleMarkerOverlay();
            markers.Markers.Add(GetMarker(features[0], "/theme/marker_blue_shadow.png"));
            markers.Markers.Add(GetMarker(features[1], "/theme/marker_green_shadow.png"));
            markers.Markers.Add(GetMarker(features[2], "/theme/marker_red_shadow.png"));

            Map1.MapUnit = GeographyUnit.DecimalDegree;
            Map1.Overlays.Add("MarkerOverlay", markers);
            Map1.CurrentExtent = new RectangleShape(-20, 20, 20, -20);

            timerOpen.Start();
            timerJammed.Start();
        }

        void timerOpen_Tick(object sender, EventArgs e)
        {
            HideTimerHandler(Status.Open);
        }

        void timerJammed_Tick(object sender, EventArgs e)
        {
            HideTimerHandler(Status.Jammed);
        }

        private void HideTimerHandler(Status status)
        {
            Collection<Marker> deleteMarkers = new Collection<Marker>();

            SimpleMarkerOverlay markerOverlay = (SimpleMarkerOverlay)Map1.Overlays["MarkerOverlay"];
            foreach (Marker marker in markerOverlay.Markers)
            {
                Status markerStatus = (Status)marker.Tag;
                if (markerStatus == status)
                {
                    deleteMarkers.Add(marker);
                }
            }

            foreach (Marker marker in deleteMarkers)
            {
                marker.Visibility = Visibility.Collapsed;
            }

            markerOverlay.Refresh();

            DispatcherTimer restoreTimer = new DispatcherTimer();
            restoreTimer.Interval = TimeSpan.FromSeconds(1);
            restoreTimer.Tick += new EventHandler(delegate
            {
                foreach (Marker marker in deleteMarkers)
                {
                    marker.Visibility = Visibility.Visible;
                }
                markerOverlay.Refresh();
                restoreTimer.Stop();
            });

            restoreTimer.Start();
        }

        private static Marker GetMarker(Feature feature, string iconPath)
        {
            Marker marker = new Marker((PointShape)feature.GetShape());
            marker.ImageSource = new BitmapImage(new Uri(iconPath, UriKind.RelativeOrAbsolute));
            marker.Tag = Enum.Parse(typeof(Status), feature.ColumnValues["Status"], true);
            return marker;
        }

        private static Collection<Feature> GetFeatures()
        {
            Collection<Feature> features = new Collection<Feature>();
            features.Add(GetFeature(Status.Open, 0, 0));
            features.Add(GetFeature(Status.Closed, 10, 10));
            features.Add(GetFeature(Status.Jammed, -10, -10));
            return features;
        }

        private static Feature GetFeature(Status status, double x, double y)
        {
            Feature feature = new Feature(new PointShape(x, y));
            feature.ColumnValues.Add("Status", status.ToString());
            return feature;
        }
    }

    public enum Status
    {
        Closed = 0,
        Open = 1,
        Jammed = 2
    }
}
